#pragma once

#include <string>
#include <vector>

#include <boost/property_tree/ptree.hpp>

#include <fcf/log.hpp>

namespace application {

  struct server_configuration_t;
  struct endpoint_configuration_t;

  struct handler_configuration_t{
    int                                     serverControlPort;
    std::string                             user;
    std::string                             password;
    int                                     listenQueue;
    int                                     clientSocketTimeout;
    int                                     maxRequestTimeout;
    int                                     serverSocketTimeout;
    int                                     maxResponseTimeout;
    int                                     startTimeout;
    int                                     stopTimeout;
    int                                     keepAliveTimeout;

    std::string                             host;
    std::string                             path;
    int                                     processes;
    int                                     processesLeakProtection;
    std::string                             script;
    int                                     maxMemory;
    int                                     maxMemoryLeakProtection;
    std::vector<int>                        dataPorts;
    std::vector<int>                        controlPorts;
    std::vector< std::vector<std::string> > nodeOptions;
    void set_values(boost::property_tree::ptree& a_ptree, endpoint_configuration_t& a_owner);  
  };

  struct endpoint_configuration_t {
    int                                   serverControlPort;
    std::string                           user;
    std::string                           password;
    int                                   listenQueue;
    int                                   clientSocketTimeout;
    int                                   maxRequestTimeout;
    int                                   serverSocketTimeout;
    int                                   maxResponseTimeout;
    int                                   startTimeout;
    int                                   stopTimeout;
    int                                   keepAliveTimeout;
    std::string                           bind4 = "none";
    std::string                           bind6 = "none";
    int                                   port  = 80;
    std::vector<handler_configuration_t>  handlers;

    void set_values(boost::property_tree::ptree& a_ptree, server_configuration_t& a_owner);
  };

  struct server_configuration_t {
    std::string                           logFile;
    int                                   logLevel = fcf::LOG_LEVEL_LOG;
    int                                   logLifeTime = 30;
    int                                   serverControlPort = 3731;

    std::string                           user;
    std::string                           password;
    bool                                  slave = false;
    int                                   listenQueue = 128;
    int                                   clientSocketTimeout = 5;
    int                                   maxRequestTimeout = 300;
    int                                   serverSocketTimeout = 5;
    int                                   maxResponseTimeout = 300;
    int                                   startTimeout = 30;
    int                                   stopTimeout = 30;
    int                                   keepAliveTimeout = 5;
    std::vector<endpoint_configuration_t> endpoints;

    void set_values(boost::property_tree::ptree& a_ptree);
  };


}
